import time
from inventorhatmini import InventorHATMini, NUM_GPIOS, LED_GPIO_1
from ioexpander import IN  # or IN_PU of a pull-up is wanted

"""
Shows how to initialise the 4 GPIO headers of Inventor HAT Mini as inputs and read them.

Press "User" to exit the program.
"""

# Constants
BRIGHTNESS = 0.4                            # The brightness of the LEDs
GPIO_NAMES = ("GP0", "GP1", "GP2", "GP3")   # Friendly names to give the GPIOs
USE_LEDS = True                             # Whether to use the LEDs to show GPIO state (requires code to run with sudo)

# Create a new InventorHATMini
board = InventorHATMini(init_leds=USE_LEDS)

# Setup each GPIO as an input
for i in range(NUM_GPIOS):
    board.gpio_pin_mode(i, IN)  # or IN_PU of a pull-up is wanted

# Read the GPIOs until the user button is pressed
while not board.switch_pressed():

    # Read each GPIO in turn and print its value
    for i in range(NUM_GPIOS):
        value = board.gpio_pin_value(i)
        print(GPIO_NAMES[i], " = ", value, sep="", end=", ")

        # Set the neighbouring LED to a colour based on
        # the input, with Green for high and Blue for low
        if value:
            board.leds.set_hsv(i + LED_GPIO_1, 0.333, 1.0, BRIGHTNESS)
        else:
            board.leds.set_hsv(i + LED_GPIO_1, 0.666, 1.0, BRIGHTNESS)

    # Print a new line
    print()

    time.sleep(0.1)

# Turn off the LED bars
board.leds.clear()



import time
import math
import random
from inventorhatmini import InventorHATMini, SERVO_1

"""
An example of how to move a servo smoothly between random positions.

Press "User" to exit the program.
"""

# Constants
UPDATES = 50            # How many times to update Servos per second
UPDATE_RATE = 1 / UPDATES
TIME_FOR_EACH_MOVE = 2  # The time to travel between each random value
UPDATES_PER_MOVE = TIME_FOR_EACH_MOVE * UPDATES

SERVO_EXTENT = 70       # How far from zero to move the servo
USE_COSINE = True       # Whether or not to use a cosine path between values

# Create a new InventorHATMini and get a servo from it
board = InventorHATMini(init_leds=False)
s = board.servos[SERVO_1]

# Get the initial value and create a random end value between the extents
start_value = s.mid_value()
end_value = random.uniform(-SERVO_EXTENT, SERVO_EXTENT)


update = 0


# Sleep until a specific time in the future. Use this instead of time.sleep() to correct for
# inconsistent timings when dealing with complex operations or external communication
def sleep_until(end_time):
    time_to_sleep = end_time - time.monotonic()
    if time_to_sleep > 0.0:
        time.sleep(time_to_sleep)


# Continually move the servo until the user button is pressed
while not board.switch_pressed():

    # Record the start time of this loop
    start_time = time.monotonic()

    # Calculate how far along this movement to be
    percent_along = update / UPDATES_PER_MOVE

    if USE_COSINE:
        # Move the servo between values using cosine
        s.to_percent(math.cos(percent_along * math.pi), 1.0, -1.0, start_value, end_value)
    else:
        # Move the servo linearly between values
        s.to_percent(percent_along, 0.0, 1.0, start_value, end_value)

    # Print out the value the servo is now at
    print("Value = ", round(s.value(), 3), sep="")

    # Move along in time
    update += 1

    # Have we reached the end of this movement?
    if update >= UPDATES_PER_MOVE:
        # Reset the counter
        update = 0

        # Set the start as the last end and create a new random end value
        start_value = end_value
        end_value = random.uniform(-SERVO_EXTENT, SERVO_EXTENT)

    # Sleep until the next update, accounting for how long the above operations took to perform
    sleep_until(start_time + UPDATE_RATE)

# Disable the servo
s.disable()



#!/usr/bin/env python3

import time

import RPi.GPIO as GPIO
from ioexpander import ADC, SuperIOE
from ioexpander.common import NORMAL_DIR
from ioexpander.encoder import MMME_CPR, ROTARY_CPR, Encoder
from ioexpander.motor import Motor, MotorState
from ioexpander.servo import Servo

from inventorhatmini.errors import NO_I2C, NO_IOE_MSG
from inventorhatmini.plasma import DummyPlasma, Plasma

__version__ = '0.0.1'


# Index Constants
MOTOR_A = 0
MOTOR_B = 1

SERVO_1 = 0
SERVO_2 = 1
SERVO_3 = 2
SERVO_4 = 3

GPIO_1 = 0
GPIO_2 = 1
GPIO_3 = 2
GPIO_4 = 3

LED_SERVO_1 = 0
LED_SERVO_2 = 1
LED_SERVO_3 = 2
LED_SERVO_4 = 3
LED_GPIO_1 = 4
LED_GPIO_2 = 5
LED_GPIO_3 = 6
LED_GPIO_4 = 7


# Count Constants
NUM_MOTORS = 2
NUM_SERVOS = 4
NUM_GPIOS = 4
NUM_LEDS = 8


class InventorHATMini():
    # I2C pins
    PI_I2C_SDA_PIN = 2
    PI_I2C_SCL_PIN = 3
    PI_I2C_INT_PIN = 4

    # WS2812 pin
    PI_LED_DATA_PIN = 12

    # I2S Audio pins
    PI_AMP_EN_PIN = 25

    # User switch pin
    PI_USER_SW_PIN = 26

    # UART / HC-SR04 Ultrasound pins
    PI_UART_TX_TRIG_PIN = 14
    PI_UART_RX_ECHO_PIN = 15

    IOE_ADDRESS = 0x17

    # Expander motor driver pins, via DRV8833PWP Dual H-Bridge
    IOE_MOTOR_A_PINS = (20, 19)
    IOE_MOTOR_B_PINS = (16, 15)

    # Expander motor encoder pins
    IOE_ENCODER_A_PINS = (3, 4)
    IOE_ENCODER_B_PINS = (26, 1)

    # Expander servo pins
    IOE_SERVO_PINS = (23, 24, 25, 22)

    # Expander GPIO/ADC pins
    IOE_GPIO_PINS = (14, 13, 9, 18)

    # Internal sense pins
    IOE_VOLTAGE_SENSE = 11
    IOE_CURRENT_SENSES = (7, 8)

    SHUNT_RESISTOR = 0.47

    def __init__(self, address=IOE_ADDRESS, motor_gear_ratio=50, init_motors=True, init_servos=True, init_leds=True, start_muted=False):
        """ Initialise inventor hat mini's hardware functions
        """
        self.address = address

        GPIO.setwarnings(False)
        GPIO.setmode(GPIO.BCM)

        # Setup user button
        GPIO.setup(self.PI_USER_SW_PIN, GPIO.IN, pull_up_down=GPIO.PUD_DOWN)

        # Setup amplifier enable. This mutes the audio by default
        GPIO.setup(self.PI_AMP_EN_PIN, GPIO.OUT, initial=GPIO.LOW if start_muted else GPIO.HIGH)

        self.__cpr = MMME_CPR * motor_gear_ratio
        self.__init_motors = init_motors
        self.__init_servos = init_servos
        self.reinit()

        if init_leds:
            # Setup the PixelStrip object to use with Inventor's LEDs, wrapped in a Plasma class
            self.leds = Plasma(NUM_LEDS, self.PI_LED_DATA_PIN)
        else:
            # Setup a dummy Plasma class, so examples don't need to check LED presence
            self.leds = DummyPlasma()

    def reinit(self):
        try:
            self.ioe = SuperIOE(i2c_addr=self.address, perform_reset=True)
        except TimeoutError:
            raise TimeoutError(NO_IOE_MSG) from None
        except OSError:
            raise OSError(NO_IOE_MSG) from None
        except FileNotFoundError:
            raise RuntimeError(NO_I2C) from None

        self.motors = None
        self.encoders = None
        if self.__init_motors:
            self.motors = [Motor(self.ioe, self.IOE_MOTOR_A_PINS), Motor(self.ioe, self.IOE_MOTOR_B_PINS)]
            self.encoders = [Encoder(self.ioe, 1, self.IOE_ENCODER_A_PINS, counts_per_rev=self.__cpr, count_microsteps=True),
                             Encoder(self.ioe, 2, self.IOE_ENCODER_B_PINS, counts_per_rev=self.__cpr, count_microsteps=True)]

        self.servos = None
        if self.__init_servos:
            self.servos = [Servo(self.ioe, self.IOE_SERVO_PINS[i]) for i in range(NUM_SERVOS)]

        self.ioe.set_mode(self.IOE_VOLTAGE_SENSE, ADC)
        self.ioe.set_mode(self.IOE_CURRENT_SENSES[0], ADC)
        self.ioe.set_mode(self.IOE_CURRENT_SENSES[1], ADC)

    def __del__(self):
        self.ioe.reset()
        GPIO.cleanup()

    def switch_pressed(self):
        return GPIO.input(self.PI_USER_SW_PIN) != 0

    def enable_motors(self):
        """ Enables both motors.
        """
        if self.motors is not None:
            for motor in self.motors:
                motor.enable()

    def disable_motors(self):
        """ Disables both motors, allowing them to spin freely.
        """
        if self.motors is not None:
            for motor in self.motors:
                motor.disable()

    def read_voltage(self):
        return (self.ioe.input(self.IOE_VOLTAGE_SENSE) * (10 + 3.9)) / 3.9

    def read_motor_current(self, motor):
        if motor < 0 or motor >= NUM_MOTORS:
            raise ValueError("motor out of range. Expected MOTOR_A (0) or MOTOR_B (1)")

        return self.ioe.input(self.IOE_CURRENT_SENSES[motor]) / self.SHUNT_RESISTOR

    def mute_audio(self):
        GPIO.output(self.PI_AMP_EN_PIN, False)

    def unmute_audio(self):
        GPIO.output(self.PI_AMP_EN_PIN, True)

    def gpio_pin_mode(self, gpio, mode=None):
        if gpio < 0 or gpio >= NUM_GPIOS:
            raise ValueError("gpio out of range. Expected GPIO_1 (0), GPIO_2 (1), GPIO_3 (2) or GPIO_4 (3)")

        if mode is None:
            return self.ioe.get_mode(self.IOE_GPIO_PINS[gpio])
        else:
            self.ioe.set_mode(self.IOE_GPIO_PINS[gpio], mode)

    def gpio_pin_value(self, gpio, value=None):
        if gpio < 0 or gpio >= NUM_GPIOS:
            raise ValueError("gpio out of range. Expected GPIO_1 (0), GPIO_2 (1), GPIO_3 (2) or GPIO_4 (3)")

        if value is None:
            return self.ioe.input(self.IOE_GPIO_PINS[gpio])
        else:
            self.ioe.output(self.IOE_GPIO_PINS[gpio], value)

    def servo_pin_mode(self, servo, mode=None):
        if self.servos is not None:
            raise ValueError("servo pin is already in use by a Servo")

        if servo < 0 or servo >= NUM_SERVOS:
            raise ValueError("servo out of range. Expected SERVO_1 (0), SERVO_2 (1), SERVO_3 (2) or SERVO_4 (3)")

        if mode is None:
            return self.ioe.get_mode(self.IOE_SERVO_PINS[servo])
        else:
            self.ioe.set_mode(self.IOE_SERVO_PINS[servo], mode)

    def servo_pin_value(self, servo, value=None, load=True, wait_for_load=False):
        if self.servos is not None:
            raise ValueError("servo pin is already in use by a Servo")

        if servo < 0 or servo >= NUM_SERVOS:
            raise ValueError("servo out of range. Expected SERVO_1 (0), SERVO_2 (1), SERVO_3 (2) or SERVO_4 (3)")

        if value is None:
            return self.ioe.input(self.IOE_SERVO_PINS[servo])
        else:
            self.ioe.output(self.IOE_SERVO_PINS[servo], value, load=load, wait_for_load=wait_for_load)

    def servo_pin_load(self, servo, wait_for_load=True):
        if self.servos is not None:
            raise ValueError("servo pin is already in use by a Servo")

        if servo < 0 or servo >= NUM_SERVOS:
            raise ValueError("servo out of range. Expected SERVO_1 (0), SERVO_2 (1), SERVO_3 (2) or SERVO_4 (3)")

        module = self.ioe.get_pwm_module(self.IOE_SERVO_PINS[servo])
        self.ioe.pwm_load(module, wait_for_load)

    def servo_pin_frequency(self, servo, frequency, load=True, wait_for_load=True):
        if self.servos is not None:
            raise ValueError("servo pin is already in use by a Servo")

        if servo < 0 or servo >= NUM_SERVOS:
            raise ValueError("servo out of range. Expected SERVO_1 (0), SERVO_2 (1), SERVO_3 (2) or SERVO_4 (3)")

        module = self.ioe.get_pwm_module(self.IOE_SERVO_PINS[servo])
        self.ioe.set_pwm_frequency(frequency, module, load=load, wait_for_load=wait_for_load)

    def encoder_from_gpio_pins(self, channel, gpio_a, gpio_b, direction=NORMAL_DIR, counts_per_rev=ROTARY_CPR, count_microsteps=False):
        if self.encoders is not None:
            if channel == 1:
                raise ValueError("channel 1 is already in use by Motor A's encoder.")
            if channel == 2:
                raise ValueError("channel 1 is already in use by Motor B's encoder.")

        if channel < 1 or channel > 4:
            raise ValueError("channel out of range. Expected 1 to 4.")

        if gpio_a < 0 or gpio_a >= NUM_GPIOS:
            raise ValueError("gpio_a out of range. Expected GPIO_1 (0), GPIO_2 (1), GPIO_3 (2) or GPIO_4 (3)")

        if gpio_b < 0 or gpio_b >= NUM_GPIOS:
            raise ValueError("gpio_b out of range. Expected GPIO_1 (0), GPIO_2 (1), GPIO_3 (2) or GPIO_4 (3)")

        return Encoder(self.ioe, channel, (self.IOE_GPIO_PINS[gpio_a], self.IOE_GPIO_PINS[gpio_b]), direction=direction, counts_per_rev=counts_per_rev, count_microsteps=count_microsteps)

    def motor_from_servo_pins(self, servo_p, servo_n, direction=NORMAL_DIR, speed_scale=MotorState.DEFAULT_SPEED_SCALE, zeropoint=MotorState.DEFAULT_ZEROPOINT,
                              deadzone=MotorState.DEFAULT_DEADZONE, freq=MotorState.DEFAULT_FREQUENCY, mode=MotorState.DEFAULT_DECAY_MODE):
        if self.servos is not None:
            raise ValueError("servo pins are already in use by Servos")

        if servo_p < 0 or servo_p >= NUM_SERVOS:
            raise ValueError("servo_p out of range. Expected SERVO_1 (0), SERVO_2 (1), SERVO_3 (2) or SERVO_4 (3)")

        if servo_n < 0 or servo_n >= NUM_SERVOS:
            raise ValueError("servo_n out of range. Expected SERVO_1 (0), SERVO_2 (1), SERVO_3 (2) or SERVO_4 (3)")

        return Motor(self.ioe, (self.IOE_SERVO_PINS[servo_p], self.IOE_SERVO_PINS[servo_n]), direction=direction,
                     speed_scale=speed_scale, zeropoint=zeropoint, deadzone=deadzone, freq=freq, mode=mode)

    def activate_watchdog(self):
        self.ioe.activate_watchdog()

    def deactivate_watchdog(self):
        self.ioe.deactivate_watchdog()

    def feed_watchdog(self):
        self.ioe.reset_watchdog_counter()

    def watchdog_timeout_occurred(self):
        return self.ioe.watchdog_timeout_occurred()

    def clear_watchdog_timeout(self):
        self.ioe.clear_watchdog_timeout()

    def is_watchdog_active(self):
        return self.ioe.is_watchdog_active()

    def set_watchdog_control(self, divider):
        self.ioe.set_watchdog_control(divider)


if __name__ == "__main__":
    board = InventorHATMini(start_muted=True)

    print("Inventor HAT Mini Function Test")

    last_state = False
    while True:
        state = board.switch_pressed()
        if last_state is not state:
            if state:
                print("User Switch pressed")
                board.unmute_audio()
                for i in range(NUM_LEDS):
                    board.leds.set_rgb(i, 0, 255, 0, show=False)
                board.leds.show()
                board.motors[MOTOR_A].full_positive()
                board.servos[SERVO_1].value(20)
            else:
                print("User Switch released")
                board.mute_audio()
                for i in range(NUM_LEDS):
                    board.leds.set_rgb(i, 255, 0, 0, show=False)
                board.leds.show()
                board.motors[MOTOR_A].coast()
                board.servos[SERVO_1].value(-20)

        last_state = state

        time.sleep(0.01)



import time
import RPi.GPIO as GPIO
from inventorhatmini import InventorHATMini, NUM_LEDS

"""
Control a HC-SR04 style ultrasonic distance sensor using the UART header on InventorHATMini.

Press "User" to exit the program.
"""

# Constants
UPDATES = 10                            # How many times to update the motor per second
UPDATE_RATE = 1 / UPDATES

LED_HUE = 0.333                         # The hue (between 0 and 1) to set the RGB LEDs to
MAX_LED_DISTANCE = 100.0                # The distance that will cause all the LEDs to be lit (in cm)
BRIGHTNESS = 0.4                        # The brightness of the RGB LED
USE_LEDS = True                         # Whether to show the distance on the LEDs (requires code to run with sudo)

# Speed of sound is 343m/s which we need in cm/ns for our distance measure
SPEED_OF_SOUND_CM_NS = 343 * 100 / 1E9  # 0.0000343 cm / ns

# Create a new InventorHATMini
board = InventorHATMini(init_leds=USE_LEDS)

# Setup the UART pins for ultrasonic sensor use
GPIO.setup(InventorHATMini.PI_UART_TX_TRIG_PIN, GPIO.OUT)
GPIO.setup(InventorHATMini.PI_UART_RX_ECHO_PIN, GPIO.IN)


# Function for reading a distance value from a connected ultrasound sensor
# Ported from github.com/pimoroni/trilobot-python
def read_distance(timeout=50, samples=3, offset=190000):
    """ Return a distance in cm from the ultrasound sensor.
    timeout: total time in ms to try to get distance reading
    samples: determines how many readings to average
    offset: Time in ns the measurement takes (prevents over estimates)
    The default offset here is about right for a Raspberry Pi 4.

    Returns the measured distance in centimetres as a float.

    To give more stable readings, this method will attempt to take several
    readings and return the average distance. You can set the maximum time
    you want it to take before returning a result so you have control over
    how long this method ties up your program. It takes as many readings
    up to the requested number of samples set as it can before the timeout
    total is reached. It then returns the average distance measured. Any
    readings where the single reading takes more than the timeout is
    ignored so these do not distort the average distance measured. If no
    valid readings are taken before the timeout then it returns zero.
    You can choose parameters to get faster but less accurate readings or
    take longer to get more samples to average before it returns. The
    timeout effectively limits the maximum distance the sensor can measure
    because if the sound pulse takes longer to return over the distance
    than the timeout set then this method returns zero rather than waiting.
    So to extend the distance that can be measured, use a larger timeout.
    """

    # Start timing
    start_time = time.perf_counter_ns()
    time_elapsed = 0
    count = 0  # Track now many samples taken
    total_pulse_durations = 0
    distance = -999

    # Loop until the timeout is exceeded or all samples have been taken
    while (count < samples) and (time_elapsed < timeout * 1000000):
        # Trigger
        GPIO.output(InventorHATMini.PI_UART_TX_TRIG_PIN, 1)
        time.sleep(.00001)  # 10 microseconds
        GPIO.output(InventorHATMini.PI_UART_TX_TRIG_PIN, 0)

        # Wait for the ECHO pin to go high
        # wait for the pulse rise
        GPIO.wait_for_edge(InventorHATMini.PI_UART_RX_ECHO_PIN, GPIO.RISING, timeout=timeout)
        pulse_start = time.perf_counter_ns()

        # And wait for it to fall
        GPIO.wait_for_edge(InventorHATMini.PI_UART_RX_ECHO_PIN, GPIO.FALLING, timeout=timeout)
        pulse_end = time.perf_counter_ns()

        # get the duration
        pulse_duration = pulse_end - pulse_start - offset
        if pulse_duration < 0:
            pulse_duration = 0  # Prevent negative readings when offset was too high

        # Only count reading if achieved in less than timeout total time
        if pulse_duration < timeout * 1000000:
            # Convert to distance and add to total
            total_pulse_durations += pulse_duration
            count += 1

        time_elapsed = time.perf_counter_ns() - start_time

    # Calculate average distance in cm if any successful reading were made
    if count > 0:
        # Calculate distance using speed of sound divided by number of samples and half
        # that as sound pulse travels from robot to obstacle and back (twice the distance)
        distance = total_pulse_durations * SPEED_OF_SOUND_CM_NS / (2 * count)

    return distance


# Continually read the distance until the user button is pressed
while not board.switch_pressed():

    distance = read_distance()
    print("Distance is {:.1f} cm".format(distance))

    for i in range(NUM_LEDS):
        if distance >= i * (MAX_LED_DISTANCE / NUM_LEDS):
            board.leds.set_hsv(i, LED_HUE, 1.0, BRIGHTNESS)

    time.sleep(UPDATE_RATE)

# Turn off the LEDs
board.leds.clear()



import sys

import mock
import pytest


@pytest.fixture(scope='function', autouse=False)
def GPIO():
    """Mock RPi.GPIO module."""
    RPi = mock.MagicMock()
    sys.modules['RPi'] = RPi
    sys.modules['RPi.GPIO'] = RPi.GPIO
    yield RPi.GPIO
    del sys.modules['RPi']
    del sys.modules['RPi.GPIO']


@pytest.fixture(scope='function', autouse=False)
def ioexpander():
    """Mock ioexpander module."""
    io_expander = mock.MagicMock()
    sys.modules['ioexpander'] = io_expander
    sys.modules['ioexpander.motor'] = io_expander.motor
    sys.modules['ioexpander.servo'] = io_expander.servo
    sys.modules['ioexpander.encoder'] = io_expander.encoder
    sys.modules['ioexpander.common'] = io_expander.common
    yield io_expander
    del sys.modules['ioexpander']


@pytest.fixture(scope='function', autouse=False)
def rpi_ws281x():
    """Mock rpi_ws281x module."""
    rpi_ws281x = mock.MagicMock()
    sys.modules['rpi_ws281x'] = rpi_ws281x
    yield rpi_ws281x
    del sys.modules['rpi_ws281x']



import time
import math
from inventorhatmini import InventorHATMini

"""
Demonstrates how to control all of the servos on Inventor HAT Mini.
"""

# Create a new InventorHATMini
board = InventorHATMini(init_leds=False)

# Enable all servos (this puts them at the middle)
for s in board.servos:
    s.enable()
time.sleep(2)

# Go to min
for s in board.servos:
    s.to_min()
time.sleep(2)

# Go to max
for s in board.servos:
    s.to_max()
time.sleep(2)

# Go back to mid
for s in board.servos:
    s.to_mid()
time.sleep(2)

SWEEPS = 3              # How many sweeps of the servo to perform
STEPS = 10              # The number of discrete sweep steps
STEPS_INTERVAL = 0.5    # The time in seconds between each step of the sequence
SWEEP_EXTENT = 70.0     # How far from zero to move the servo when sweeping

# Do a sine sweep
for j in range(SWEEPS):
    for i in range(360):
        value = math.sin(math.radians(i)) * SWEEP_EXTENT
        for s in board.servos:
            s.value(value)
        time.sleep(0.02)

# Do a stepped sweep
for j in range(SWEEPS):
    for i in range(0, STEPS):
        for s in board.servos:
            s.to_percent(i, 0, STEPS, 0.0 - SWEEP_EXTENT, SWEEP_EXTENT)
        time.sleep(STEPS_INTERVAL)
    for i in range(0, STEPS):
        for s in board.servos:
            s.to_percent(i, STEPS, 0, 0.0 - SWEEP_EXTENT, SWEEP_EXTENT)
        time.sleep(STEPS_INTERVAL)

# Disable the servos
for s in board.servos:
    s.disable()



import time
from inventorhatmini import InventorHATMini, NUM_GPIOS, LED_GPIO_1
from ioexpander import OUT

"""
Shows how to initialise and 4 GPIO headers of Inventor HAT Mini as outputs and set them.

Press "User" to exit the program.
"""

# Constants
BRIGHTNESS = 0.4                            # The brightness of the LEDs
GPIO_NAMES = ("GP0", "GP1", "GP2", "GP3")   # Friendly names to give the GPIOs
USE_LEDS = True                             # Whether to use the LEDs to show GPIO state (requires code to run with sudo)

# Create a new InventorHATMini
board = InventorHATMini(init_leds=USE_LEDS)

# The GPIO to set to high
current_gpio = 0

# Setup each GPIO as an output
for i in range(NUM_GPIOS):
    board.gpio_pin_mode(i, OUT)

# Set the GPIOs until the user button is pressed
while not board.switch_pressed():

    # Set each GPIO in turn and print its value
    for i in range(NUM_GPIOS):
        # Set the pin to high if this is the current gpio pin, otherwise low
        value = (i == current_gpio)
        board.gpio_pin_value(i, value)
        print(GPIO_NAMES[i], " = ", value, sep="", end=", ")

        # Set the neighbouring LED to a colour based on
        # the output, with Green for high and Blue for low
        if value:
            board.leds.set_hsv(i + LED_GPIO_1, 0.333, 1.0, BRIGHTNESS)
        else:
            board.leds.set_hsv(i + LED_GPIO_1, 0.666, 1.0, BRIGHTNESS)

    # Print a new line
    print()

    # Advance the current gpio, and wrap if necessary
    current_gpio += 1
    if current_gpio >= NUM_GPIOS:
        current_gpio = 0

    time.sleep(0.5)

# Set all the gpio pins back to low
for i in range(NUM_GPIOS):
    board.gpio_pin_value(i, False)

# Turn off the LED bars
board.leds.clear()



import time
from inventorhatmini import InventorHATMini, NUM_SERVOS, LED_SERVO_1
from ioexpander import OUT

"""
Shows how to initialise and use Inventor HAT Mini's servo headers as 3.3V outputs and set them.

Press "User" to exit the program.
"""

# Constants
BRIGHTNESS = 0.4                        # The brightness of the LEDs
SERVO_NAMES = ("S0", "S1", "S2", "S3")  # Friendly names to give the servos
USE_LEDS = True                         # Whether to use the LEDs to show servo state (requires code to run with sudo)

# Create a new InventorHATMini
board = InventorHATMini(init_servos=False, init_leds=USE_LEDS)

# The servo to set to high
current_servo = 0

# Setup each servo as an output
for i in range(NUM_SERVOS):
    board.servo_pin_mode(i, OUT)

# Set the servos until the user button is pressed
while not board.switch_pressed():

    # Set each servos in turn and print its value
    for i in range(NUM_SERVOS):
        # Set the pin to high if this is the current servo pin, otherwise low
        value = (i == current_servo)
        board.servo_pin_value(i, value)
        print(SERVO_NAMES[i], " = ", value, sep="", end=", ")

        # Set the neighbouring LED to a colour based on
        # the output, with Green for high and Blue for low
        if value:
            board.leds.set_hsv(i + LED_SERVO_1, 0.333, 1.0, BRIGHTNESS)
        else:
            board.leds.set_hsv(i + LED_SERVO_1, 0.666, 1.0, BRIGHTNESS)

    # Print a new line
    print()

    # Advance the current servo, and wrap if necessary
    current_servo += 1
    if current_servo >= NUM_SERVOS:
        current_servo = 0

    time.sleep(0.5)

# Set all the servo pins back to low
for i in range(NUM_SERVOS):
    board.servo_pin_value(i, False)

# Turn off the LED bars
board.leds.clear()



import time
from inventorhatmini import InventorHATMini, NUM_MOTORS

"""
Shows how to read the internal sensors of Inventor HAT Mini.

Press "User" to exit the program.
"""

# Constants
MOTOR_NAMES = ["A", "B"]      # Friendly names to give the encoders

# Create a new InventorHATMini
board = InventorHATMini(init_leds=False)

# Set the motors spinning so that a current can be measured
for motor in board.motors:
    motor.duty(1.0)

# Read the internal sensors until the user button is pressed
while not board.switch_pressed():

    # Read the voltage sense and print the value
    voltage = board.read_voltage()
    print("V =", round(voltage, 4), end=", ")

    # Read the current sense of each motor and print the value
    for i in range(NUM_MOTORS):
        current = board.read_motor_current(i)
        print(f"Current {MOTOR_NAMES[i]} = ", round(current, 4), end=", ")

    # Print a new line
    print()

    time.sleep(0.5)

# Disable the motors
for motor in board.motors:
    motor.disable()



import time
import math
from inventorhatmini import InventorHATMini, NUM_MOTORS, MOTOR_A, MOTOR_B, LED_SERVO_1, LED_GPIO_4
from ioexpander.common import PID, REVERSED_DIR

"""
A demonstration of driving both of Inventor HAT Mini's motor outputs between
positions, with the help of their attached encoders and PID control.

Press "User" to exit the program.
"""

# Constants
ENCODER_NAMES = ["A", "B"]              # Friendly names to give the encoders
GEAR_RATIO = 50                         # The gear ratio of the motors
SPEED_SCALE = 5.4                       # The scaling to apply to each motor's speed to match its real-world speed

UPDATES = 100                           # How many times to update the motor per second
UPDATE_RATE = 1 / UPDATES
TIME_FOR_EACH_MOVE = 2                  # The time to travel between each value
UPDATES_PER_MOVE = TIME_FOR_EACH_MOVE * UPDATES
PRINT_DIVIDER = 4                       # How many of the updates should be printed (i.e. 2 would be every other update)

# LED constant
BRIGHTNESS = 0.4                        # The brightness of the RGB LED
USE_LEDS = True                         # Whether to show a wave on the LEDs (requires code to run with sudo)

# PID values
POS_KP = 0.14                           # Position proportional (P) gain
POS_KI = 0.0                            # Position integral (I) gain
POS_KD = 0.0022                         # Position derivative (D) gain


# Create a new InventorHATMini
board = InventorHATMini(motor_gear_ratio=GEAR_RATIO, init_leds=USE_LEDS)

# Set the speed scale of the motors
board.motors[MOTOR_A].speed_scale(SPEED_SCALE)
board.motors[MOTOR_B].speed_scale(SPEED_SCALE)

# Reverse the direction of the left motor and encoder
board.motors[MOTOR_A].direction(REVERSED_DIR)
board.encoders[MOTOR_A].direction(REVERSED_DIR)

# Create PID objects for position control
pos_pids = [PID(POS_KP, POS_KI, POS_KD, UPDATE_RATE) for i in range(NUM_MOTORS)]

# Enable all motors
for m in board.motors:
    m.enable()


update = 0
print_count = 0

# Set the initial and end values
start_value = 0.0
end_value = 270.0

captures = [None] * NUM_MOTORS


# Sleep until a specific time in the future. Use this instead of time.sleep() to correct for
# inconsistent timings when dealing with complex operations or external communication
def sleep_until(end_time):
    time_to_sleep = end_time - time.monotonic()
    if time_to_sleep > 0.0:
        time.sleep(time_to_sleep)


# Continually move the motor until the user button is pressed
while not board.switch_pressed():

    # Record the start time of this loop
    start_time = time.monotonic()

    # Capture the state of all the encoders
    for i in range(NUM_MOTORS):
        captures[i] = board.encoders[i].capture()

    # Calculate how far along this movement to be
    percent_along = min(update / UPDATES_PER_MOVE, 1.0)

    for i in range(NUM_MOTORS):
        # Move the motor between values using cosine
        pos_pids[i].setpoint = (((-math.cos(percent_along * math.pi) + 1.0) / 2.0) * (end_value - start_value)) + start_value

        # Calculate the velocity to move the motor closer to the position setpoint
        vel = pos_pids[i].calculate(captures[i].degrees, captures[i].degrees_per_second)

        # Set the new motor driving speed
        board.motors[i].speed(vel)

    # Update the LEDs
    board.leds.set_hsv(LED_SERVO_1, percent_along, 1.0, BRIGHTNESS)
    board.leds.set_hsv(LED_GPIO_4, percent_along, 1.0, BRIGHTNESS)

    # Print out the current motor values and their setpoints, but only on every multiple
    if print_count == 0:
        for i in range(NUM_MOTORS):
            print(ENCODER_NAMES[i], "=", captures[i].degrees, end=", ")
        print()

    # Increment the print count, and wrap it
    print_count = (print_count + 1) % PRINT_DIVIDER

    update += 1     # Move along in time

    # Have we reached the end of this movement?
    if update >= UPDATES_PER_MOVE:
        update = 0  # Reset the counter

        # Swap the start and end values
        temp = start_value
        start_value = end_value
        end_value = temp

    # Sleep until the next update, accounting for how long the above operations took to perform
    sleep_until(start_time + UPDATE_RATE)

# Stop all the motors
for m in board.motors:
    m.disable()

# Turn off the LEDs
board.leds.clear()



import time
from inventorhatmini import InventorHATMini, MOTOR_A, MOTOR_B
from ioexpander.motor import SLOW_DECAY  # , FAST_DECAY

"""
A fun example of how to change a motor's frequency to have it play a song.

It uses code written by Avram Piltch - check out his Tom's Hardware article!
https://www.tomshardware.com/uk/how-to/buzzer-music-raspberry-pi-pico

Press "User" to start or stop the song.
"""

# This handy dictionary converts notes into frequencies
TONES = {
    "B0": 31,
    "C1": 33,
    "CS1": 35,
    "D1": 37,
    "DS1": 39,
    "E1": 41,
    "F1": 44,
    "FS1": 46,
    "G1": 49,
    "GS1": 52,
    "A1": 55,
    "AS1": 58,
    "B1": 62,
    "C2": 65,
    "CS2": 69,
    "D2": 73,
    "DS2": 78,
    "E2": 82,
    "F2": 87,
    "FS2": 93,
    "G2": 98,
    "GS2": 104,
    "A2": 110,
    "AS2": 117,
    "B2": 123,
    "C3": 131,
    "CS3": 139,
    "D3": 147,
    "DS3": 156,
    "E3": 165,
    "F3": 175,
    "FS3": 185,
    "G3": 196,
    "GS3": 208,
    "A3": 220,
    "AS3": 233,
    "B3": 247,
    "C4": 262,
    "CS4": 277,
    "D4": 294,
    "DS4": 311,
    "E4": 330,
    "F4": 349,
    "FS4": 370,
    "G4": 392,
    "GS4": 415,
    "A4": 440,
    "AS4": 466,
    "B4": 494,
    "C5": 523,
    "CS5": 554,
    "D5": 587,
    "DS5": 622,
    "E5": 659,
    "F5": 698,
    "FS5": 740,
    "G5": 784,
    "GS5": 831,
    "A5": 880,
    "AS5": 932,
    "B5": 988,
    "C6": 1047,
    "CS6": 1109,
    "D6": 1175,
    "DS6": 1245,
    "E6": 1319,
    "F6": 1397,
    "FS6": 1480,
    "G6": 1568,
    "GS6": 1661,
    "A6": 1760,
    "AS6": 1865,
    "B6": 1976,
    "C7": 2093,
    "CS7": 2217,
    "D7": 2349,
    "DS7": 2489,
    "E7": 2637,
    "F7": 2794,
    "FS7": 2960,
    "G7": 3136,
    "GS7": 3322,
    "A7": 3520,
    "AS7": 3729,
    "B7": 3951,
    "C8": 4186,
    "CS8": 4435,
    "D8": 4699,
    "DS8": 4978
}

# Put the notes for your song in here!
SONG = ("F6", "F6", "E6", "F6", "F5", "P", "F5", "P", "C6", "AS5", "A5", "C6", "F6", "P", "F6", "P",
        "G6", "FS6", "G6", "G5", "P", "G5", "P", "G6", "F6", "E6", "D6", "C6", "P", "C6", "P", "D6",
        "E6", "F6", "E6", "D6", "C6", "D6", "C6", "AS5", "A5", "AS5", "A5", "G5", "F5", "G5", "F5", "E5",
        "D5", "C5", "D5", "E5", "F5", "G5", "AS5", "A5", "G5", "A5", "F5", "P", "F5")

NOTE_DURATION = 0.150           # The time (in seconds) to play each note for. Change this to make the song play faster or slower
STATIONARY_TOGGLE = 0.002       # The time (in seconds) between each direction switch of the motor when using STATIONARY_PLAYBACK
STATIONARY_PLAYBACK = False     # Whether to play the song with or without the motors spinning
DECAY_MODE = SLOW_DECAY         # The motor decay mode to use, either FAST_DECAY (0) or SLOW_DECAY (1). Affects the song's quality


# Create a new InventorHATMini
board = InventorHATMini(init_leds=False)
motor_a = board.motors[MOTOR_A]
motor_b = board.motors[MOTOR_B]


# Variables for recording the button state and if it has been toggled
# Starting as True makes the song play automatically
button_toggle = True
last_button_state = False


# Function to check if the button has been toggled
def check_button_toggle():
    global button_toggle
    global last_button_state
    button_state = board.switch_pressed()
    if button_state and not last_button_state:
        button_toggle = not button_toggle
    last_button_state = button_state

    return button_toggle


# Sleep until a specific time in the future. Use this instead of time.sleep() to correct for
# inconsistent timings when dealing with complex operations or external communication
def sleep_until(end_time):
    time_to_sleep = end_time - time.monotonic()
    if time_to_sleep > 0.0:
        time.sleep(time_to_sleep)

print("Press the 'User' button on your Inventor HAT Mini to start or stop the song.")

while True:
    # Has the button been toggled?
    if check_button_toggle():

        # Play the song
        for i in range(len(SONG)):
            # Record the start time of this loop
            start_time = time.monotonic()

            if check_button_toggle():
                if SONG[i] == "P":
                    # This is a "pause" note, so stop the motors
                    motor_a.stop()
                    motor_b.stop()
                    sleep_until(start_time + NOTE_DURATION)
                else:
                    # Get the frequency of the note and set the motors to it
                    freq = TONES[SONG[i]]
                    motor_a.frequency(freq)
                    motor_b.frequency(freq)

                    if STATIONARY_PLAYBACK:
                        # Set the motors to 50% duty cycle to play the note, but alternate
                        # the direction so that the motor does not actually spin
                        t = 0
                        while t < NOTE_DURATION:
                            motor_a.duty(0.5)
                            motor_b.duty(0.5)
                            t += STATIONARY_TOGGLE
                            sleep_until(start_time + t)

                            motor_a.duty(-0.5)
                            motor_b.duty(-0.5)
                            t += STATIONARY_TOGGLE
                            sleep_until(start_time + t)
                    else:
                        # Set the motors to 50% duty cycle to play the note whilst spinning
                        motor_a.duty(0.5)
                        motor_b.duty(0.5)
                        sleep_until(start_time + NOTE_DURATION)
            else:
                # The button was toggled again, so stop playing the song
                break

        button_toggle = False

        # The song has finished, so disable the motors
        motor_a.disable()
        motor_b.disable()

    time.sleep(0.01)



import time
import math
from inventorhatmini import InventorHATMini, NUM_MOTORS

"""
A demonstration of using Inventor HAT Mini's internal watchdog to stop motors
from spinning when a simulated code lock-up occurs, by causing a board reset,
and recovering from it.

Press "User" to simulate a lock-up.
Stop the program using the stop button in Thonny or Ctrl+C in the Terminal
"""

# Constants
SPEED = 5             # The speed that the motors will cycle at
UPDATES = 50          # How many times to update motors per second
UPDATE_RATE = 1 / UPDATES
SPEED_EXTENT = 1.0    # How far from zero to drive the motors

# Create a new InventorHATMini
board = InventorHATMini(init_leds=False)


# Sleep until a specific time in the future. Use this instead of time.sleep() to correct for
# inconsistent timings when dealing with complex operations or external communication
def sleep_until(end_time):
    time_to_sleep = end_time - time.monotonic()
    if time_to_sleep > 0.0:
        time.sleep(time_to_sleep)


# Run in an infinite loop to show watchdog reset and recovery
while True:
    offset = 0.0

    # Configure and activate the watchdog
    board.set_watchdog_control(256)  # Accepts a multiple of two (except 4), up to 256
    board.activate_watchdog()
    print("Watchdog active")

    # Make waves until a watchdog reset occurs
    while not board.watchdog_timeout_occurred():

        # Record the start time of this loop
        start_time = time.monotonic()

        # Keep the watchdog fed so it does not reset the board
        board.feed_watchdog()

        offset += SPEED / 1000.0

        # Update both motors
        for i in range(NUM_MOTORS):
            angle = (i + offset) * math.pi
            board.motors[i].speed(math.sin(angle) * SPEED_EXTENT)

        # Put the code into a loop here whilst the user switch is pressed
        switch_pressed = False
        while board.switch_pressed():
            if not switch_pressed:
                print("Switch pressed")
                switch_pressed = True

        # Sleep until the next update, accounting for how long the above operations took to perform
        sleep_until(start_time + UPDATE_RATE)

    # The watchdog has reset the board
    print("Reset occurred")

    # Reinitialise the parts of InventorHATMini that were reset
    board.reinit()
    print("Reinitialising...")
    print()



import pygame
from inventorhatmini import InventorHATMini

"""
Play a WAV audio file from your Inventor HAT Mini, using PyGame!

Press "User" to play the sound.

Note: WAV files need to be signed 16-bit
"""

# Create a new InventorHATMini
board = InventorHATMini(init_leds=False)

# Initialise PyGame so we can play sounds with it
pygame.init()

# Load the sound from file
ahoy = pygame.mixer.Sound("./ahoy.wav")

print("Press the 'User' button on your Inventor HAT Mini to play the sound.")

# Loop forever
while True:
    if board.switch_pressed():
        # Play the sound
        channel = ahoy.play()
        print("Playing ... ", end="")

        # Wait for the sound to finish playing
        while channel.get_busy():
            pass

        print("Done")



import time
import subprocess
from inventorhatmini import InventorHATMini

"""
Play a WAV audio file from your Inventor HAT Mini, using Aplay!

Press "User" to play the sound.

Note: WAV files need to be signed 16-bit
"""

# Create a new InventorHATMini
board = InventorHATMini(init_leds=False)

print("Press the 'User' button on your Inventor HAT Mini to play the sound.")

# Loop forever
while True:
    if board.switch_pressed():
        # Play the sound
        print("Playing ... ", end="")

        # Use aplay to play the sound
        command = "aplay -q ./ahoy.wav &"
        subprocess.run(command, shell=True)

        time.sleep(0.25)
        print("Done")



import time
import math
from inventorhatmini import InventorHATMini, NUM_SERVOS, LED_SERVO_1

"""
An example of applying a wave pattern to a group of servos and the LEDs.

Press "User" to exit the program.
"""

# Constants
SPEED = 5             # The speed that the LEDs will cycle at
BRIGHTNESS = 0.4      # The brightness of the LEDs
UPDATES = 50          # How many times to update LEDs and Servos per second
UPDATE_RATE = 1 / UPDATES
SERVO_EXTENT = 70.0   # How far from zero to move the servos
USE_LEDS = True       # Whether to show a wave on the LEDs (requires code to run with sudo)

# Create a new InventorHATMini
board = InventorHATMini(init_leds=USE_LEDS)

offset = 0.0


# Sleep until a specific time in the future. Use this instead of time.sleep() to correct for
# inconsistent timings when dealing with complex operations or external communication
def sleep_until(end_time):
    time_to_sleep = end_time - time.monotonic()
    if time_to_sleep > 0.0:
        time.sleep(time_to_sleep)


# Make waves until the user button is pressed
while not board.switch_pressed():

    # Record the start time of this loop
    start_time = time.monotonic()

    offset += SPEED / 1000.0

    # Update all the Servos
    for i in range(NUM_SERVOS):
        angle = ((i / NUM_SERVOS) + offset) * math.pi
        board.servos[i].value(math.sin(angle) * SERVO_EXTENT)

        # Read back the servo's angle and use that to set a hue on the neighbouring LED
        hue = ((board.servos[i].value() / SERVO_EXTENT) + 1) * 0.333
        board.leds.set_hsv(i + LED_SERVO_1, hue, 1.0, BRIGHTNESS)

    # Sleep until the next update, accounting for how long the above operations took to perform
    sleep_until(start_time + UPDATE_RATE)

# Stop all the servos
for s in board.servos:
    s.disable()

# Turn off the LEDs
board.leds.clear()



def test_setup(GPIO, rpi_ws281x, ioexpander):
    import inventorhatmini
    inventorhatmini.InventorHATMini()
    GPIO.setwarnings.assert_called_once_with(False)
    GPIO.setmode.assert_called_once_with(GPIO.BCM)



import time
from inventorhatmini import InventorHATMini, NUM_GPIOS, LED_GPIO_1
from ioexpander import ADC

"""
Shows how to initialise and read the 4 ADC headers of Inventor HAT Mini.

Press "User" to exit the program.
"""

# Constants
BRIGHTNESS = 0.4                        # The brightness of the LEDs
UPDATES = 10                            # How many times to update LEDs per second
ADC_NAMES = ("A0", "A1", "A2", "A3")    # Friendly names to give the ADCs
USE_LEDS = True                         # Whether to use the LEDs to show ADC state (requires code to run with sudo)

# Create a new InventorHATMini
board = InventorHATMini(init_leds=USE_LEDS)

# Setup each GPIO as an analog input
for i in range(NUM_GPIOS):
    board.gpio_pin_mode(i, ADC)

# Read the ADCs until the user button is pressed
while not board.switch_pressed():

    # Read each ADC in turn and print its voltage
    for i in range(NUM_GPIOS):
        voltage = board.gpio_pin_value(i)
        print(ADC_NAMES[i], " = ", round(voltage, 3), sep="", end=", ")

        # Set the neighbouring LED to a colour based on the
        # voltage, with Green for high and Blue for low
        hue = (2.0 - (voltage / 3.3)) * 0.333
        board.leds.set_hsv(i + LED_GPIO_1, hue, 1.0, BRIGHTNESS)

    # Print a new line
    print()

    time.sleep(1.0 / UPDATES)

# Turn off the LED bars
board.leds.clear()



import time
import math
from inventorhatmini import InventorHATMini

"""
Demonstrates how to control both motors on Inventor HAT Mini.
"""

# Create a new InventorHATMini
board = InventorHATMini(init_leds=False)

# Enable all motors
for m in board.motors:
    m.enable()
time.sleep(2)

# Drive at full positive
for m in board.motors:
    m.full_positive()
time.sleep(2)

# Stop moving
for m in board.motors:
    m.stop()
time.sleep(2)

# Drive at full negative
for m in board.motors:
    m.full_negative()
time.sleep(2)

# Coast to a gradual stop
for m in board.motors:
    m.coast()
time.sleep(2)


SWEEPS = 2              # How many speed sweeps of the motors to perform
STEPS = 10              # The number of discrete sweep steps
STEPS_INTERVAL = 0.5    # The time in seconds between each step of the sequence
SPEED_EXTENT = 1.0      # How far from zero to drive the motors when sweeping

# Do a sine speed sweep
for j in range(SWEEPS):
    for i in range(360):
        speed = math.sin(math.radians(i)) * SPEED_EXTENT
        for m in board.motors:
            m.speed(speed)
        time.sleep(0.02)

# Do a stepped speed sweep
for j in range(SWEEPS):
    for i in range(0, STEPS):
        for m in board.motors:
            m.to_percent(i, 0, STEPS, 0.0 - SPEED_EXTENT, SPEED_EXTENT)
        time.sleep(STEPS_INTERVAL)
    for i in range(0, STEPS):
        for m in board.motors:
            m.to_percent(i, STEPS, 0, 0.0 - SPEED_EXTENT, SPEED_EXTENT)
        time.sleep(STEPS_INTERVAL)

# Disable the motors
for m in board.motors:
    m.disable()



import time
import math
from inventorhatmini import InventorHATMini, SERVO_2, SERVO_3
from ioexpander.common import NORMAL_DIR  # REVERSED_DIR

"""
Shows how to use Inventor HAT Mini's Servo headers to control an externally connected motor driver.

Note that servo pins on the same PWM module must be used. These are:
- SERVO_1 and SERVO_4
- SERVO_2 and SERVO_3

Press "User" to exit the program.
"""

# Constants
DIRECTION = NORMAL_DIR  # The direction to spin the motor in. NORMAL_DIR (0), REVERSED_DIR (1)
SPEED = 5               # The speed that the motor will cycle at
UPDATES = 50            # How many times to update the motor per second
UPDATE_RATE = 1 / UPDATES
SPEED_EXTENT = 1.0      # How far from zero to drive the motor

# Create a new InventorHATMini
board = InventorHATMini(init_servos=False, init_leds=False)

# Create an Encoder object using two GPIO pins
motor = board.motor_from_servo_pins(SERVO_2, SERVO_3, direction=DIRECTION)


offset = 0.0


# Sleep until a specific time in the future. Use this instead of time.sleep() to correct for
# inconsistent timings when dealing with complex operations or external communication
def sleep_until(end_time):
    time_to_sleep = end_time - time.monotonic()
    if time_to_sleep > 0.0:
        time.sleep(time_to_sleep)


# Make waves until the user button is pressed
while not board.switch_pressed():

    # Record the start time of this loop
    start_time = time.monotonic()

    offset += SPEED / 1000.0

    # Update the motor
    angle = offset * math.pi
    motor.speed(math.sin(angle) * SPEED_EXTENT)

    # Sleep until the next update, accounting for how long the above operations took to perform
    sleep_until(start_time + UPDATE_RATE)

# Stop the motor
motor.disable()



import time
from inventorhatmini import InventorHATMini, MOTOR_A
from ioexpander.common import PID, NORMAL_DIR  # , REVERSED_DIR

"""
A program to aid in the discovery and tuning of motor PID
values for velocity control. It does this by commanding the
motor to drive repeatedly between two setpoint speeds and
plots the measured response.

Press "User" to exit the program.
"""

# Constants
GEAR_RATIO = 50                         # The gear ratio of the motor
DIRECTION = NORMAL_DIR                  # The direction to spin the motor in. NORMAL_DIR (0), REVERSED_DIR (1)
SPEED_SCALE = 5.4                       # The scaling to apply to the motor's speed to match its real-world speed

UPDATES = 100                           # How many times to update the motor per second
UPDATE_RATE = 1 / UPDATES
PRINT_WINDOW = 0.25                     # The time (in seconds) after a new setpoint, to display print out motor values
MOVEMENT_WINDOW = 2.0                   # The time (in seconds) between each new setpoint being set
PRINT_DIVIDER = 1                       # How many of the updates should be printed (i.e. 2 would be every other update)

# Multipliers for the different printed values, so they appear nicely on the Thonny plotter
ACC_PRINT_SCALE = 0.01                  # Acceleration multiplier

VELOCITY_EXTENT = 3                     # How far from zero to drive the motor at, in revolutions per second

# PID values
VEL_KP = 30.0                           # Velocity proportional (P) gain
VEL_KI = 0.0                            # Velocity integral (I) gain
VEL_KD = 0.4                            # Velocity derivative (D) gain


# Create a new InventorHATMini and get a motor and encoder from it
board = InventorHATMini(motor_gear_ratio=GEAR_RATIO, init_leds=False)
m = board.motors[MOTOR_A]
enc = board.encoders[MOTOR_A]

# Set the motor's speed scale
m.speed_scale(SPEED_SCALE)

# Set the motor and encoder's direction
m.direction(DIRECTION)
enc.direction(DIRECTION)

# Create PID object for velocity control
vel_pid = PID(VEL_KP, VEL_KI, VEL_KD, UPDATE_RATE)

# Enable the motor to get started
m.enable()

# Set the initial setpoint velocity
vel_pid.setpoint = VELOCITY_EXTENT


update = 0
print_count = 0


# Sleep until a specific time in the future. Use this instead of time.sleep() to correct for
# inconsistent timings when dealing with complex operations or external communication
def sleep_until(end_time):
    time_to_sleep = end_time - time.monotonic()
    if time_to_sleep > 0.0:
        time.sleep(time_to_sleep)


# Continually move the motor until the user button is pressed
while not board.switch_pressed():

    # Record the start time of this loop
    start_time = time.monotonic()

    # Capture the state of the encoder
    capture = enc.capture()

    # Calculate the acceleration to apply to the motor to move it closer to the velocity setpoint
    accel = vel_pid.calculate(capture.revolutions_per_second)

    # Accelerate or decelerate the motor
    m.speed(m.speed() + (accel * UPDATE_RATE))

    # Print out the current motor values and their setpoints,
    # but only for the first few updates and only every multiple
    if update < (PRINT_WINDOW * UPDATES) and print_count == 0:
        print("Vel =", capture.revolutions_per_second, end=", ")
        print("Vel SP =", vel_pid.setpoint, end=", ")
        print("Accel =", accel * ACC_PRINT_SCALE, end=", ")
        print("Speed =", m.speed())

    # Increment the print count, and wrap it
    print_count = (print_count + 1) % PRINT_DIVIDER

    update += 1     # Move along in time

    # Have we reached the end of this time window?
    if update >= (MOVEMENT_WINDOW * UPDATES):
        update = 0  # Reset the counter

        # Set the new velocity setpoint to be the inverse of the current setpoint
        vel_pid.setpoint = 0.0 - vel_pid.setpoint

    # Sleep until the next update, accounting for how long the above operations took to perform
    sleep_until(start_time + UPDATE_RATE)

# Disable the motor
m.disable()



import time
import math
from inventorhatmini import InventorHATMini, MOTOR_A

"""
Demonstrates how to control a motor on Inventor HAT Mini.
"""

# Create a new InventorHATMini
board = InventorHATMini(init_leds=False)

# Access the motor from Inventor and enable it
m = board.motors[MOTOR_A]
m.enable()
time.sleep(2)

# Drive at full positive
m.full_positive()
time.sleep(2)

# Stop moving
m.stop()
time.sleep(2)

# Drive at full negative
m.full_negative()
time.sleep(2)

# Coast to a gradual stop
m.coast()
time.sleep(2)


SWEEPS = 2              # How many speed sweeps of the motor to perform
STEPS = 10              # The number of discrete sweep steps
STEPS_INTERVAL = 0.5    # The time in seconds between each step of the sequence
SPEED_EXTENT = 1.0      # How far from zero to drive the motor when sweeping

# Do a sine speed sweep
for j in range(SWEEPS):
    for i in range(360):
        m.speed(math.sin(math.radians(i)) * SPEED_EXTENT)
        time.sleep(0.02)

# Do a stepped speed sweep
for j in range(SWEEPS):
    for i in range(0, STEPS):
        m.to_percent(i, 0, STEPS, 0.0 - SPEED_EXTENT, SPEED_EXTENT)
        time.sleep(STEPS_INTERVAL)
    for i in range(0, STEPS):
        m.to_percent(i, STEPS, 0, 0.0 - SPEED_EXTENT, SPEED_EXTENT)
        time.sleep(STEPS_INTERVAL)

# Disable the motor
m.disable()



import time
from inventorhatmini import InventorHATMini, NUM_MOTORS  # , MOTOR_A, MOTOR_B
# from ioexpander.common import REVERSED_DIR

"""
Demonstrates how to read the angles of Inventor HAT Mini's two motor encoders.

Press "User" to exit the program.
"""

# Constants
NAMES = ["A", "B"]      # Friendly names to give the encoders
GEAR_RATIO = 50         # The gear ratio of the motor

# Create a new InventorHATMini
board = InventorHATMini(motor_gear_ratio=GEAR_RATIO, init_leds=False)

# Uncomment the below lines (and the top imports) to
# reverse the counting direction of an encoder
# board.encoders[MOTOR_A].direction(REVERSED_DIR)
# board.encoders[MOTOR_B].direction(REVERSED_DIR)

# Read the encoders until the user button is pressed
while not board.switch_pressed():

    # Print out the angle of each encoder
    for i in range(NUM_MOTORS):
        print(NAMES[i], "=", board.encoders[i].degrees(), end=", ")
    print()

    time.sleep(0.1)



from colorsys import hsv_to_rgb

from rpi_ws281x import PixelStrip

from inventorhatmini.errors import LED_INIT_FAILED


class Plasma():
    LED_FREQ_HZ = 800000  # LED signal frequency in hertz (usually 800khz)
    LED_DMA = 10          # DMA channel to use for generating signal (try 10)
    LED_BRIGHTNESS = 255  # Set to 0 for darkest and 255 for brightest
    LED_INVERT = False    # True to invert the signal (when using NPN transistor level shift)
    LED_CHANNEL = 0       # set to '1' for GPIOs 13, 19, 41, 45 or 53
    LED_GAMMA = [
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2,
        2, 2, 2, 3, 3, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5,
        6, 6, 6, 7, 7, 7, 8, 8, 8, 9, 9, 9, 10, 10, 11, 11,
        11, 12, 12, 13, 13, 13, 14, 14, 15, 15, 16, 16, 17, 17, 18, 18,
        19, 19, 20, 21, 21, 22, 22, 23, 23, 24, 25, 25, 26, 27, 27, 28,
        29, 29, 30, 31, 31, 32, 33, 34, 34, 35, 36, 37, 37, 38, 39, 40,
        40, 41, 42, 43, 44, 45, 46, 46, 47, 48, 49, 50, 51, 52, 53, 54,
        55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70,
        71, 72, 73, 74, 76, 77, 78, 79, 80, 81, 83, 84, 85, 86, 88, 89,
        90, 91, 93, 94, 95, 96, 98, 99, 100, 102, 103, 104, 106, 107, 109, 110,
        111, 113, 114, 116, 117, 119, 120, 121, 123, 124, 126, 128, 129, 131, 132, 134,
        135, 137, 138, 140, 142, 143, 145, 146, 148, 150, 151, 153, 155, 157, 158, 160,
        162, 163, 165, 167, 169, 170, 172, 174, 176, 178, 179, 181, 183, 185, 187, 189,
        191, 193, 194, 196, 198, 200, 202, 204, 206, 208, 210, 212, 214, 216, 218, 220,
        222, 224, 227, 229, 231, 233, 235, 237, 239, 241, 244, 246, 248, 250, 252, 255]

    def __init__(self, num_leds, pin):
        # Setup the PixelStrip object to use with Inventor's LEDs
        self.leds = PixelStrip(num_leds, pin, self.LED_FREQ_HZ, self.LED_DMA, self.LED_INVERT, self.LED_BRIGHTNESS, self.LED_CHANNEL, self.LED_GAMMA)
        try:
            # Attempt to initialise the library
            self.leds.begin()
            self.leds.show()
        except RuntimeError:
            raise RuntimeError(LED_INIT_FAILED) from None

    def set_rgb(self, index, r, g, b, show=True):
        if index < 0 or index >= self.leds.numPixels():
            raise ValueError("index out of range. Expected 0 to NUM_LEDs - 1")

        self.leds.setPixelColorRGB(index, r, g, b)

        if show:
            self.leds.show()

    def set_hsv(self, index, h, s=1.0, v=1.0, show=True):
        if index < 0 or index >= self.leds.numPixels():
            raise ValueError("index out of range. Expected 0 to NUM_LEDs - 1")

        r, g, b = [int(c * 255) for c in hsv_to_rgb(h, s, v)]
        self.leds.setPixelColorRGB(index, r, g, b)

        if show:
            self.leds.show()

    def get(self, index):
        if index < 0 or index >= self.leds.numPixels():
            raise ValueError("index out of range. Expected 0 to NUM_LEDs - 1")

        return self.leds.getPixelColorRGB(index)

    def clear(self, show=True):
        for i in range(self.leds.numPixels()):
            self.leds.setPixelColorRGB(i, 0, 0, 0)

        if show:
            self.leds.show()

    def show(self):
        self.leds.show()


class DummyPlasma():
    def __init__(self):
        pass

    def set_rgb(self, index, r, g, b, show=True):
        pass

    def set_hsv(self, index, h, s=1.0, v=1.0, show=True):
        pass

    def get(self, index):
        pass

    def clear(self, show=True):
        pass

    def show(self):
        pass



import time
from inventorhatmini import InventorHATMini, MOTOR_A, NUM_LEDS
from ioexpander.common import PID, NORMAL_DIR  # , REVERSED_DIR

"""
A demonstration of how a motor with an encoder can be used
as a programmable rotary encoder for user input, with
force-feedback for arbitrary detents and end stops.

Press "User" to exit the program.
"""

# Constants
GEAR_RATIO = 50                         # The gear ratio of the motor
DIRECTION = NORMAL_DIR                  # The direction to spin the motor in. NORMAL_DIR (0), REVERSED_DIR (1)
SPEED_SCALE = 5.4                       # The scaling to apply to the motor's speed to match its real-world speed

UPDATES = 100                           # How many times to update the motor per second
UPDATE_RATE = 1 / UPDATES

DETENT_SIZE = 20                        # The size (in degrees) of each detent region
MIN_DETENT = 0                          # The minimum detent that can be counted to
MAX_DETENT = NUM_LEDS - 1               # The maximum detent that can be counted to
MAX_DRIVE_PERCENT = 0.5                 # The maximum drive force (as a percent) to apply when crossing detents

BRIGHTNESS = 0.4                        # The brightness of the RGB LED
USE_LEDS = True                         # Whether to show the encoder position on the LEDs (requires code to run with sudo)

# PID values
POS_KP = 0.14                           # Position proportional (P) gain
POS_KI = 0.0                            # Position integral (I) gain
POS_KD = 0.0022                         # Position derivative (D) gain


# Create a new InventorHATMini and get a motor and encoder from it
board = InventorHATMini(motor_gear_ratio=GEAR_RATIO, init_leds=USE_LEDS)
m = board.motors[MOTOR_A]
enc = board.encoders[MOTOR_A]

# Set the motor's speed scale
m.speed_scale(SPEED_SCALE)

# Set the motor and encoder's direction
m.direction(DIRECTION)
enc.direction(DIRECTION)

# Create PID object for position control
pos_pid = PID(POS_KP, POS_KI, POS_KD, UPDATE_RATE)

# Enable the motor to get started
m.enable()


current_detent = 0


# Function to deal with a detent change
def detent_change(change):
    global current_detent

    # Update the current detent and pid setpoint
    current_detent += change
    pos_pid.setpoint = (current_detent * DETENT_SIZE)     # Update the motor position setpoint
    print("Detent =", current_detent)

    # Convert the current detent to a hue and set the onboard led to it
    hue = (current_detent - MIN_DETENT) / (MAX_DETENT - MIN_DETENT)
    for i in range(NUM_LEDS):
        if current_detent == i:
            board.leds.set_hsv(i, hue, 1.0, BRIGHTNESS)
        else:
            board.leds.set_hsv(i, hue, 1.0, 0.0)


# Call the function once to set the setpoint and print the value
detent_change(0)


# Sleep until a specific time in the future. Use this instead of time.sleep() to correct for
# inconsistent timings when dealing with complex operations or external communication
def sleep_until(end_time):
    time_to_sleep = end_time - time.monotonic()
    if time_to_sleep > 0.0:
        time.sleep(time_to_sleep)


# Continually move the motor until the user button is pressed
while not board.switch_pressed():

    # Record the start time of this loop
    start_time = time.monotonic()

    # Capture the state of the encoder
    capture = enc.capture()

    # Get the current detent's centre angle
    detent_angle = (current_detent * DETENT_SIZE)

    # Is the current angle above the region of this detent?
    if capture.degrees > detent_angle + (DETENT_SIZE / 2):
        # Is there another detent we can move to?
        if current_detent < MAX_DETENT:
            detent_change(1)    # Increment to the next detent

    # Is the current angle below the region of this detent?
    elif capture.degrees < detent_angle - (DETENT_SIZE / 2):
        # Is there another detent we can move to?
        if current_detent > MIN_DETENT:
            detent_change(-1)    # Decrement to the next detent

    # Calculate the velocity to move the motor closer to the position setpoint
    vel = pos_pid.calculate(capture.degrees, capture.degrees_per_second)

    # If the current angle is within the detent range, limit the max vel
    # (aka feedback force) that the user will feel when turning the motor between detents
    if (capture.degrees >= MIN_DETENT * DETENT_SIZE) and (capture.degrees <= MAX_DETENT * DETENT_SIZE):
        vel = max(min(vel, MAX_DRIVE_PERCENT), -MAX_DRIVE_PERCENT)

    # Set the new motor driving speed
    m.speed(vel)

    # Sleep until the next update, accounting for how long the above operations took to perform
    sleep_until(start_time + UPDATE_RATE)

# Disable the motor
m.disable()

# Turn off the LEDs
board.leds.clear()



import time
import random
from inventorhatmini import InventorHATMini
from ttastromech import TTAstromech

"""
Make your Inventor HAT Mini sound like an Astromech Droid!

This example uses the ttastromech library, for more details see https://pypi.org/project/ttastromech/
To install enter the following on the command line: pip install ttastromech

Press "User" to play a random word out of the speaker.
"""

# Constants
MIN_WORD_LENGTH = 3     # The smallest word size
MAX_WORD_LENGTH = 10    # The largest word size
WAIT_TIME = 0.25        # The time (in seconds) to wait after playing a word, before accepting user input again

# Create a new InventorHATMini and TTAstromech
board = InventorHATMini(init_leds=False)
droid = TTAstromech()


# Generate the characters from `c1` to `c2`, inclusive.
def char_range(c1, c2):
    for c in range(ord(c1), ord(c2) + 1):
        yield chr(c)


# Generate a random word from the letters a to z.
def random_string(word_length=5, no_repeat=True):
    chars = list(char_range("a", "z"))
    word = ""
    char_count = 0
    while char_count != word_length:
        char = chars[random.randint(0, 25)]
        if char in word and no_repeat:
            pass
        else:
            word += char
            char_count += 1
    print("Saying:", word)
    return word


print("Press the 'User' button on your Inventor HAT Mini to play the sound.")

# Loop forever
while True:
    if board.switch_pressed():
        # Speak a random word when "User" is pressed
        word_length = random.randint(MIN_WORD_LENGTH, MAX_WORD_LENGTH)
        droid.speak(random_string(word_length, no_repeat=False))
        time.sleep(WAIT_TIME)



from inventorhatmini import InventorHATMini, SERVO_1, SERVO_2, SERVO_3, SERVO_4
from ioexpander.servo import Calibration, ANGULAR, LINEAR, CONTINUOUS

"""
Shows how to configure Inventor HAT Mini's servos with different
common calibrations, as well as a completely custom one.
"""

# Create a new InventorHATMini and access its four servos
board = InventorHATMini(init_leds=False)
angular_servo = board.servos[SERVO_1]
linear_servo = board.servos[SERVO_2]
continuous_servo = board.servos[SERVO_3]
custom_servo = board.servos[SERVO_4]


# -----------------------------------------------------
# Modify the calibration of an angular servo
# -----------------------------------------------------

# Access the calibration of the first servo and print it out
cal = angular_servo.calibration()
print("Angular Servo:", cal, end="\n\n")


WIDE_ANGLE_RANGE = 270  # The range we want the anglular servo to cover

# Lets modify the calibration to increase its range
cal.apply_default_pairs(ANGULAR)
cal.first_value(-WIDE_ANGLE_RANGE / 2)
cal.last_value(WIDE_ANGLE_RANGE / 2)

# Now apply the modified calibration to the servo and confirm it worked
angular_servo.calibration(cal)
print("Wide Angle Servo:", angular_servo.calibration(), end="\n\n")


# ---------------------------------------------------
# Create and modify the calibration of a linear servo
# ---------------------------------------------------

LINEAR_RANGE = 50  # The range we want the linear servo to cover

# Update the linear servo so its max value matches the real distance travelled
cal = linear_servo.calibration()
cal.apply_default_pairs(LINEAR)
cal.last_value(LINEAR_RANGE)

# Apply the modified calibration to the servo and confirm it worked
linear_servo.calibration(cal)
print("Linear Servo:", linear_servo.calibration(), end="\n\n")


# ----------------------------------------------------------------
# Create and modify the calibration of a continuous rotation servo
# ----------------------------------------------------------------

CONTINUOUS_SPEED = 10  # The speed we want the continuous servo to cover

# Update the continuous rotation servo so its value matches its real speed
cal = continuous_servo.calibration()
cal.apply_default_pairs(CONTINUOUS)
cal.first_value(-CONTINUOUS_SPEED)
cal.last_value(CONTINUOUS_SPEED)

# Apply the modified calibration to the servo and confirm it worked
continuous_servo.calibration(cal)
print("Continuous Servo:", continuous_servo.calibration(), end="\n\n")


# ------------------------------------------------------
# Create a custom calibration and build a servo using it
# ------------------------------------------------------

# Create an empty calibration
cal = Calibration()

# Give it a range of -45 to 45 degrees, corresponding to pulses of 1000 and 2000 microseconds
cal.apply_two_pairs(1000, 2000, -45, 45)

# Turn off the lower and upper limits, so the servo can go beyond 45 degrees
cal.limit_to_calibration(False, False)

# Create a servo on pin 3 using the custom calibration and confirmed it worked
custom_servo.calibration(cal)
print("Custom Servo:", custom_servo.calibration(), end="\n\n")



NO_IOE_MSG = "Could not communicate with the IO Expander. Check the HAT is properly attached and that the I2C pins of the Raspberry Pi are not being used for another function."
NO_I2C = "The I2C interface needed for communication with the IO Expander is disabled. To enable it either run `sudo raspi-config nonint do_i2c 0` in the terminal, or use the `Raspberry Pi Configuration` UI accessible under `Preferences`."
LED_INIT_FAILED = """Failed to initialise the RGB LEDs because higher privileges are needed to control the necessary hardware of the Raspberry Pi.
To overcome this, either:
  - [if using the terminal] run your code with `sudo` in front.
  - [if using Thonny] close it, then open it via the terminal using `sudo thonny`. Then run your code as normal.
  - disable LED initialisation by including the parameter `init_leds=False` when creating your InventorHATMini object. This will prevent the LEDs from being used later on in your code."""



import time
import math
from inventorhatmini import InventorHATMini, SERVO_1

"""
Demonstrates how to control a single servo on Inventor HAT Mini.
"""

# Create a new InventorHATMini
board = InventorHATMini(init_leds=False)

# Access the servo from Inventor and enable it (this puts it at the middle)
s = board.servos[SERVO_1]
s.enable()
time.sleep(2)

# Go to min
s.to_min()
time.sleep(2)

# Go to max
s.to_max()
time.sleep(2)

# Go back to mid
s.to_mid()
time.sleep(2)


SWEEPS = 3              # How many sweeps of the servo to perform
STEPS = 10              # The number of discrete sweep steps
STEPS_INTERVAL = 0.5    # The time in seconds between each step of the sequence
SWEEP_EXTENT = 70.0     # How far from zero to move the servo when sweeping

# Do a sine sweep
for j in range(SWEEPS):
    for i in range(360):
        s.value(math.sin(math.radians(i)) * SWEEP_EXTENT)
        time.sleep(0.02)

# Do a stepped sweep
for j in range(SWEEPS):
    for i in range(0, STEPS):
        s.to_percent(i, 0, STEPS, 0.0 - SWEEP_EXTENT, SWEEP_EXTENT)
        time.sleep(STEPS_INTERVAL)
    for i in range(0, STEPS):
        s.to_percent(i, STEPS, 0, 0.0 - SWEEP_EXTENT, SWEEP_EXTENT)
        time.sleep(STEPS_INTERVAL)

# Disable the servo
s.disable()



import time
from inventorhatmini import InventorHATMini, NUM_LEDS

"""
Displays a rotating rainbow pattern on Inventor HAT Mini's onboard LED bars.

Press "User" to exit the program.
"""

# Constants
SPEED = 5           # The speed that the LEDs will cycle at
BRIGHTNESS = 0.4    # The brightness of the LEDs
UPDATES = 50        # How many times the LEDs will be updated per second
UPDATE_RATE = 1 / UPDATES
AUTO_SHOW = True    # Whether to update each LED as they are set

# Create a new InventorHATMini
board = InventorHATMini()

# Variables
offset = 0.0


# Sleep until a specific time in the future. Use this instead of time.sleep() to correct for
# inconsistent timings when dealing with complex operations or external communication
def sleep_until(end_time):
    time_to_sleep = end_time - time.monotonic()
    if time_to_sleep > 0.0:
        time.sleep(time_to_sleep)


# Make rainbows until the user button is pressed
while not board.switch_pressed():

    # Record the start time of this loop
    start_time = time.monotonic()

    offset += SPEED / 1000.0

    # Update all the LEDs
    for i in range(NUM_LEDS):
        hue = float(i) / NUM_LEDS
        board.leds.set_hsv(i, hue + offset, 1.0, BRIGHTNESS, show=AUTO_SHOW)

    # If the LEDs were not updated when they were set, update them all now
    if not AUTO_SHOW:
        board.leds.show()

    # Sleep until the next update, accounting for how long the above operations took to perform
    sleep_until(start_time + UPDATE_RATE)

# Turn off the LED bars
board.leds.clear()



import time
from inventorhatmini import InventorHATMini, MOTOR_A
from ioexpander.common import NORMAL_DIR  # , REVERSED_DIR

"""
A program that profiles the speed of a motor across its PWM
duty cycle range using the attached encoder for feedback.
"""

# Constants
GEAR_RATIO = 50                         # The gear ratio of the motor
DIRECTION = NORMAL_DIR                  # The direction to spin the motor in. NORMAL_DIR (0), REVERSED_DIR (1)
SPEED_SCALE = 5.4                       # The scaling to apply to the motor's speed. Set this to the maximum measured speed
ZERO_POINT = 0.0                        # The duty cycle that corresponds with zero speed when plotting the motor's speed as a straight line
DEAD_ZONE = 0.0                         # The duty cycle below which the motor's friction prevents it from moving

DUTY_STEPS = 100                        # How many duty cycle steps to sample the speed of
SETTLE_TIME = 0.1                       # How long to wait after changing motor duty cycle
CAPTURE_TIME = 0.2                      # How long to capture the motor's speed at each step

# Create a new InventorHATMini and get a motor and encoder from it
board = InventorHATMini(motor_gear_ratio=GEAR_RATIO, init_leds=False)
m = board.motors[MOTOR_A]
enc = board.encoders[MOTOR_A]

# Set the motor's speed scale, zeropoint, and deadzone
m.speed_scale(SPEED_SCALE)
m.zeropoint(ZERO_POINT)
m.deadzone(DEAD_ZONE)

# Set the motor and encoder's direction
m.direction(DIRECTION)
enc.direction(DIRECTION)


# Function that performs a single profiling step
def profile_at_duty(duty):
    # Set the motor to a new duty cycle and wait for it to settle
    if DIRECTION == 1:
        m.duty(0.0 - duty)
    else:
        m.duty(duty)
    time.sleep(SETTLE_TIME)

    # Perform a dummy capture to clear the encoder
    enc.capture()

    # Wait for the capture time to pass
    time.sleep(CAPTURE_TIME)

    # Perform a capture and read the measured speed
    capture = enc.capture()
    measured_speed = capture.revolutions_per_second

    # These are some alternate speed measurements from the encoder
    # measured_speed = capture.revolutions_per_minute
    # measured_speed = capture.degrees_per_second
    # measured_speed = capture.radians_per_second

    # Print out the expected and measured speeds, as well as their difference
    print("Duty = ", m.duty(), ", Expected = ", m.speed(), ", Measured = ", measured_speed, ", Diff = ", m.speed() - measured_speed, sep="")


# Enable the motor to get started
m.enable()

print("Profiler Starting...")

# Profile from 0% up to one step below 100%
for i in range(DUTY_STEPS):
    profile_at_duty(i / DUTY_STEPS)

# Profile from 100% down to one step above 0%
for i in range(DUTY_STEPS):
    profile_at_duty((DUTY_STEPS - i) / DUTY_STEPS)

# Profile from 0% down to one step above -100%
for i in range(DUTY_STEPS):
    profile_at_duty(-i / DUTY_STEPS)

# Profile from -100% up to one step below 0%
for i in range(DUTY_STEPS):
    profile_at_duty(-(DUTY_STEPS - i) / DUTY_STEPS)

# Profile 0% again
profile_at_duty(0)

print("Profiler Finished...")

# Disable the motor now the profiler has finished
m.disable()



import time
from inventorhatmini import InventorHATMini, MOTOR_A, MOTOR_B, NUM_MOTORS, NUM_LEDS
from ioexpander.common import PID, REVERSED_DIR

"""
A demonstration of driving both of Inventor HAT Mini's motor outputs through a
sequence of velocities, with the help of their attached encoders and PID control.

Press "User" to exit the program.
"""

# Wheel friendly names
LEFT = MOTOR_A
RIGHT = MOTOR_B
NAMES = ["LEFT", "RIGHT"]

# Constants
GEAR_RATIO = 50                         # The gear ratio of the motors
SPEED_SCALE = 5.4                       # The scaling to apply to each motor's speed to match its real-world speed

UPDATES = 100                           # How many times to update the motor per second
UPDATE_RATE = 1 / UPDATES
TIME_FOR_EACH_MOVE = 2                  # The time to travel between each value
UPDATES_PER_MOVE = TIME_FOR_EACH_MOVE * UPDATES
PRINT_DIVIDER = 4                       # How many of the updates should be printed (i.e. 2 would be every other update)

DRIVING_SPEED = 1.0                     # The speed to drive the wheels at, from 0.0 to SPEED_SCALE

# PID values
VEL_KP = 30.0                           # Velocity proportional (P) gain
VEL_KI = 0.0                            # Velocity integral (I) gain
VEL_KD = 0.4                            # Velocity derivative (D) gain

# LED Constants
BRIGHTNESS = 0.4                        # The brightness of the LEDs
SPEED_TO_CYCLING = 0.02 / SPEED_SCALE   # The factor to convert between motor speed and LED cycle rate
BAR_GRADIENT = 0.125                    # The percentage of the colour spectrum to have the LEDs gradient over
HALF_LEDS = NUM_LEDS // 2
USE_LEDS = True                         # Whether to show a pattern on the LEDs whilst driving (requires code to run with sudo)


# Create a new InventorHATMini
board = InventorHATMini(motor_gear_ratio=GEAR_RATIO, init_leds=USE_LEDS)

# Set the speed scale of the motors
board.motors[LEFT].speed_scale(SPEED_SCALE)
board.motors[RIGHT].speed_scale(SPEED_SCALE)

# Reverse the direction of the left motor and encoder
board.motors[LEFT].direction(REVERSED_DIR)
board.encoders[LEFT].direction(REVERSED_DIR)

# Create PID objects for position control
vel_pids = [PID(VEL_KP, VEL_KI, VEL_KD, UPDATE_RATE) for i in range(NUM_MOTORS)]


# LED Hue Variables
offset_l = 0.0
offset_r = 0.0


# Helper functions for driving in common directions
def drive_forward(speed):
    global offset_l
    global offset_r
    vel_pids[LEFT].setpoint = speed
    vel_pids[RIGHT].setpoint = speed
    offset_l += speed * SPEED_TO_CYCLING
    offset_r += speed * SPEED_TO_CYCLING


def turn_right(speed):
    global offset_l
    global offset_r
    vel_pids[LEFT].setpoint = speed
    vel_pids[RIGHT].setpoint = -speed
    offset_l += speed * SPEED_TO_CYCLING
    offset_r -= speed * SPEED_TO_CYCLING


def stop():
    vel_pids[LEFT].setpoint = 0
    vel_pids[RIGHT].setpoint = 0


# Enable the motor to get started
for m in board.motors:
    m.enable()


# Variables
update = 0
print_count = 0
sequence = 0
captures = [None] * NUM_MOTORS


# Sleep until a specific time in the future. Use this instead of time.sleep() to correct for
# inconsistent timings when dealing with complex operations or external communication
def sleep_until(end_time):
    time_to_sleep = end_time - time.monotonic()
    if time_to_sleep > 0.0:
        time.sleep(time_to_sleep)


# Continually move the motor until the user switch is pressed
while not board.switch_pressed():

    # Record the start time of this loop
    start_time = time.monotonic()

    # Capture the state of all the encoders
    for i in range(NUM_MOTORS):
        captures[i] = board.encoders[i].capture()

    for i in range(NUM_MOTORS):
        # Calculate the acceleration to apply to the motor to move it closer to the velocity setpoint
        accel = vel_pids[i].calculate(captures[i].revolutions_per_second)

        # Accelerate or decelerate the motor
        board.motors[i].speed(board.motors[i].speed() + (accel * UPDATE_RATE))

    # Print out the current motor values, but only on every multiple
    if print_count == 0:
        for i in range(NUM_MOTORS):
            print(NAMES[i], "=", captures[i].revolutions_per_second, end=", ")
        print()

    # Increment the print count, and wrap it
    print_count = (print_count + 1) % PRINT_DIVIDER

    update += 1     # Move along in time

    # Have we reached the end of this movement?
    if update >= UPDATES_PER_MOVE:
        update = 0  # Reset the counter

        # Move on to the next part of the sequence
        sequence += 1

        # Loop the sequence back around
        if sequence >= 5:
            sequence = 0

    # Set the motor speeds, based on the sequence
    if sequence == 0:
        drive_forward(DRIVING_SPEED)
    elif sequence == 1:
        drive_forward(-DRIVING_SPEED)
    elif sequence == 2:
        turn_right(DRIVING_SPEED)
    elif sequence == 3:
        turn_right(-DRIVING_SPEED)
    elif sequence == 4:
        stop()

    if offset_l < 0.0:
        offset_l += 1.0

    if offset_r < 0.0:
        offset_r += 1.0

    # Update the LED bars
    for i in range(HALF_LEDS):
        hue = (i / HALF_LEDS) * BAR_GRADIENT
        board.leds.set_hsv(i, hue + offset_l, 1.0, BRIGHTNESS)
        board.leds.set_hsv(NUM_LEDS - i - 1, hue + offset_r, 1.0, BRIGHTNESS)

    # Sleep until the next update, accounting for how long the above operations took to perform
    sleep_until(start_time + UPDATE_RATE)

# Stop all the motors
for m in board.motors:
    m.disable()

# Turn off the LED bars
board.leds.clear()



import time
import math
import random
from inventorhatmini import InventorHATMini, MOTOR_A
from ioexpander.common import PID, NORMAL_DIR  # , REVERSED_DIR

"""
An example of how to move a motor smoothly between random positions,
with the help of it's attached encoder and PID control.

Press "User" to exit the program.
"""

# Constants
GEAR_RATIO = 50                         # The gear ratio of the motor

DIRECTION = NORMAL_DIR                  # The direction to spin the motor in. NORMAL_DIR (0), REVERSED_DIR (1)
SPEED_SCALE = 5.4                       # The scaling to apply to the motor's speed to match its real-world speed

UPDATES = 100                           # How many times to update the motor per second
UPDATE_RATE = 1 / UPDATES
TIME_FOR_EACH_MOVE = 1                  # The time to travel between each random value
UPDATES_PER_MOVE = TIME_FOR_EACH_MOVE * UPDATES
PRINT_DIVIDER = 4                       # How many of the updates should be printed (i.e. 2 would be every other update)

# Multipliers for the different printed values, so they appear nicely on the Thonny plotter
SPD_PRINT_SCALE = 20                    # Driving Speed multiplier

POSITION_EXTENT = 180                   # How far from zero to move the motor, in degrees
INTERP_MODE = 2                         # The interpolating mode between setpoints. STEP (0), LINEAR (1), COSINE (2)

# PID values
POS_KP = 0.14                           # Position proportional (P) gain
POS_KI = 0.0                            # Position integral (I) gain
POS_KD = 0.0022                         # Position derivative (D) gain


# Create a new InventorHATMini and get a motor and encoder from it
board = InventorHATMini(motor_gear_ratio=GEAR_RATIO, init_leds=False)
m = board.motors[MOTOR_A]
enc = board.encoders[MOTOR_A]

# Set the motor's speed scale
m.speed_scale(SPEED_SCALE)

# Set the motor and encoder's direction
m.direction(DIRECTION)
enc.direction(DIRECTION)

# Create PID object for position control
pos_pid = PID(POS_KP, POS_KI, POS_KD, UPDATE_RATE)

# Enable the motor to get started
m.enable()


update = 0
print_count = 0

# Set the initial value and create a random end value between the extents
start_value = 0.0
end_value = random.uniform(-POSITION_EXTENT, POSITION_EXTENT)


# Sleep until a specific time in the future. Use this instead of time.sleep() to correct for
# inconsistent timings when dealing with complex operations or external communication
def sleep_until(end_time):
    time_to_sleep = end_time - time.monotonic()
    if time_to_sleep > 0.0:
        time.sleep(time_to_sleep)


# Continually move the motor until the user button is pressed
while not board.switch_pressed():

    # Record the start time of this loop
    start_time = time.monotonic()

    # Capture the state of the encoder
    capture = enc.capture()

    # Calculate how far along this movement to be
    percent_along = min(update / UPDATES_PER_MOVE, 1.0)

    if INTERP_MODE == 0:
        # Move the motor instantly to the end value
        pos_pid.setpoint = end_value
    elif INTERP_MODE == 2:
        # Move the motor between values using cosine
        pos_pid.setpoint = (((-math.cos(percent_along * math.pi) + 1.0) / 2.0) * (end_value - start_value)) + start_value
    else:
        # Move the motor linearly between values
        pos_pid.setpoint = (percent_along * (end_value - start_value)) + start_value

    # Calculate the velocity to move the motor closer to the position setpoint
    vel = pos_pid.calculate(capture.degrees, capture.degrees_per_second)

    # Set the new motor driving speed
    m.speed(vel)

    # Print out the current motor values and their setpoints, but only on every multiple
    if print_count == 0:
        print("Pos =", capture.degrees, end=", ")
        print("Pos SP =", pos_pid.setpoint, end=", ")
        print("Speed = ", m.speed() * SPD_PRINT_SCALE)

    # Increment the print count, and wrap it
    print_count = (print_count + 1) % PRINT_DIVIDER

    update += 1     # Move along in time

    # Have we reached the end of this movement?
    if update >= UPDATES_PER_MOVE:
        update = 0  # Reset the counter

        # Set the start as the last end and create a new random end value
        start_value = end_value
        end_value = random.uniform(-POSITION_EXTENT, POSITION_EXTENT)

    # Sleep until the next update, accounting for how long the above operations took to perform
    sleep_until(start_time + UPDATE_RATE)

# Disable the motor
m.disable()



import time
from inventorhatmini import InventorHATMini, GPIO_1, GPIO_2
# from ioexpander.common import REVERSED_DIR

"""
An example of how to read a rotary encoder connected to Inventor HAT Mini's GPIO pins.

Press "User" to exit the program.
"""

# Constants
CHANNEL = 3         # The encoder channel to use (1 and 2 are used by Inventor's motor connectors)

# Create a new InventorHATMini
board = InventorHATMini(init_leds=False)

# Create an Encoder object using two GPIO pins
enc = board.encoder_from_gpio_pins(CHANNEL, GPIO_1, GPIO_2)

# Uncomment the below line (and the top imports) to
# reverse the counting direction of the encoder
# enc.direction(REVERSED_DIR)

# Read the encoder until the user button is pressed
while not board.switch_pressed():

    # Print out the current step and turn of the rotary encoder
    print("Step = ", enc.step(), ", Turn = ", enc.turn(), sep="")

    time.sleep(0.1)



import time
import math
from inventorhatmini import InventorHATMini, NUM_LEDS, NUM_MOTORS

"""
An example of applying a wave pattern to Inventor HAT Mini's motors and LEDs.

Press "User" to exit the program.
"""

# Constants
SPEED = 5             # The speed that the LEDs and motors will cycle at
BRIGHTNESS = 0.4      # The brightness of the LEDs
UPDATES = 50          # How many times to update LEDs and motors per second
UPDATE_RATE = 1 / UPDATES
SPEED_EXTENT = 1.0    # How far from zero to drive the motors
HALF_LEDS = NUM_LEDS // 2
USE_LEDS = True       # Whether to show a pattern on the LEDs whilst driving (requires code to run with sudo)


# Create a new InventorHATMini
board = InventorHATMini(init_leds=USE_LEDS)

offset = 0.0


# Sleep until a specific time in the future. Use this instead of time.sleep() to correct for
# inconsistent timings when dealing with complex operations or external communication
def sleep_until(end_time):
    time_to_sleep = end_time - time.monotonic()
    if time_to_sleep > 0.0:
        time.sleep(time_to_sleep)


# Make waves until the user button is pressed
while not board.switch_pressed():

    # Record the start time of this loop
    start_time = time.monotonic()

    offset += SPEED / 1000.0

    # Update the LED bars
    for i in range(HALF_LEDS):
        hue = i / (NUM_LEDS * 4)
        board.leds.set_hsv(i, hue + offset, 1.0, BRIGHTNESS)
        board.leds.set_hsv(NUM_LEDS - i - 1, hue + offset, 1.0, BRIGHTNESS)

    # Update both motors
    for i in range(NUM_MOTORS):
        angle = (i + offset) * math.pi
        board.motors[i].speed(math.sin(angle) * SPEED_EXTENT)

    # Sleep until the next update, accounting for how long the above operations took to perform
    sleep_until(start_time + UPDATE_RATE)

# Stop both motors
for m in board.motors:
    m.disable()

# Turn off the LEDs
board.leds.clear()
